import { decodePreviewableId } from '@previewjs/analyzer-api';
import { createChromelessWorkspace } from '@previewjs/chromeless';
import { globby } from 'globby';

async function generateScreenshots({
  page,
  frameworkPlugins,
  filePathPattern,
  cwd = process.cwd(),
  generateScreenshotPath,
  onError,
  onScreenshotGenerated
}) {
  const workspace = await createChromelessWorkspace({
    frameworkPlugins,
    rootDir: cwd
  });
  const preview = await workspace.startPreview(page);
  const filePaths = await globby(filePathPattern, {
    gitignore: true,
    ignore: ["**/node_modules/**"],
    cwd,
    followSymbolicLinks: false
  });
  const { components, stories } = await workspace.crawlFiles(filePaths);
  for (const previewable of [...components, ...stories]) {
    const { filePath, name } = decodePreviewableId(previewable.id);
    try {
      await preview.show(previewable.id);
      const screenshotPath = generateScreenshotPath({ filePath, name });
      await preview.iframe.takeScreenshot(screenshotPath);
      if (onScreenshotGenerated) {
        onScreenshotGenerated({ filePath, name, screenshotPath });
      }
    } catch (e) {
      if (onError) {
        const halt = onError(e, { filePath, name });
        if (halt) {
          return;
        }
      } else {
        throw new Error(
          `Unable to generate screenshot for ${previewable.id}`,
          // https://stackoverflow.com/a/42755876
          // @ts-ignore
          { cause: e }
        );
      }
    }
  }
  await preview.stop();
  await workspace.dispose();
}

export { generateScreenshots };
